%% Math 448/548 - Example 3.4 - Farm Problem 
% This Matlab code illustrates the solution of a multivariable 
% constraint optimization using *linear programming*. 
% Executing the code *REQUIRES* access to the Optimization Toolbox!

%% 
% First identify the variables:
%%
% * x1 = acres of corn planted
% * x2 = acres of wheat planted
% * x3 = acres of oat planted
% and the constraints
%%
% * w = 3.0*x1+1.0*x2+1.5*x3 <= 1000 (irrigation required);
% * l = 0.8*x1+0.2*x2+0.3*x3 <=  300 (labor required);
% * t =     x1+    x2+    x3 <=  625 (total acreage planted);
%%
% Objective function to be maximized (total yield)
%%
% z = 400*x1+200*x2+250*x3;

%%
% Define the constraint coefficient matrix A 
% and the vectors representing the constraint inequalities/equalities
% [See help document for the *linprog* MATLAB command.]

clear all, close all, clc
f = [400;200;250]; % coeffients of linear objective function
A = [3   1   1.5;
     0.8 0.2 0.3; 
     1   1   1];
b = [1000;300;625]; % RHS of inequality constraints

%% Linear Programming (Large Scale Method)
% This is one (efficient) method for the linear programming task. Note that
% *linprog* minimizes the objective function, which means that for
% maximization one needs to replace f with -f.

format bank
[x,fval,exitflag,output,lambda] = linprog(-f,A,b);
x, fmax=-fval

%%
% Slack variables (Large Scale Method)
b-A*x

%%
% Shadow Prices 
shadowprices = lambda.ineqlin

%% Medium Scale Linear Programming (SIMPLEX METHOD)
% Alternately, one can employ the simplex method explicitly
% [Expect 'different' optimal values for x.]

options = optimset('LargeScale','off','Simplex','on');
lb=[0;0;0];
[x,fval,exitflag,output,lambda] = ...
    linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax=-fval

%%
% Slack variables (Simplex Metod)
Slack = b-A*x

%%
% Shadow prices
shadowprices=lambda.ineqlin

%%
% Notice that the two different methods yield the same maximum for the objective
% function, BUT different 'optimal' choice for the decision variables x1,x2,x3. 
% The Large Scale method gives a 'better' value for the optimal
% slack value than the Simplex Method.

%% Sensitivity analysis - One additional acre-ft of water

b = [1001;300;625];

lb=[0;0;0];
[x,fval] = linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax_new=-fval

Gain = fmax_new-fmax

Slack = b-A*x


%%
% Conclusion: Relaxing the constraint on the water resource (by 1 unit) will cause an
% increase in total yield of $100 (exacly the shadow price computed
% earlier).


%% Sensitivity analysis - higher yield for corn

f = [450;200;250];
b = [1000;300;625];

lb=[0;0;0];
[x,fval] = linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax_new=-fval
Gain = fmax_new-fmax
Slack = b-A*x
shadowprice_new = lambda.ineqlin

%%
% Conclusion: In case of a higher yield for corn ($450 instead of $400), the 
% increase in total yield is 50*x1=9375. Yhere is no change in the slack variables.
% Note the change in the shadow prices. 

%% Sensitivity analysis - higher yield for oats

f = [400;200;260];
b = [1000;300;625];

lb=[0;0;0];
[x,fval] = linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax_new=-fval 

Gain = fmax_new-fmax

Slack = b-A*x


%%
% Conclusion: In case of a higher yield for oats ($260 instead of $250), 
% there is a significant change in the 'optimal' decision variables. More 
% oats and less corn should be planted. 

%% New variety of corn

f = [400;200;250];
A = [2.5   1   1.5;
     0.8 0.2 0.3; 
     1   1   1]; 
b = [1000;300;625];

lb=[0;0;0];
[x,fval] = linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax_new=-fval 

Gain = fmax_new-fmax

Slack = b-A*x

%% New crop - barley
% Adjust the constraints to accomodate a new crop
%%
% * w = 3.0*x1+1.0*x2+1.5*x3+ 1.5*x4 <= 1000;
% * l = 0.8*x1+0.2*x2+0.3*x3+0.25*x4 <=  300;
% * t =     x1+    x2+    x3+     x4 <=  625;

f = [400;200;250;200];
A = [3   1   1.5   1.5;
     0.8 0.2 0.3  0.25; 
     1   1   1  1]; 
b = [1000;300;625];

lb=[0;0;0;0];
[x,fval] = linprog(-f,A,b,[],[],lb,[],[],options);
x, fmax_new=-fval 

Slack = b-A*x
